;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: soonfits.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, March 23, 1997
;
; Modified: Sun Mar 23 17:36:55 1997 by meisner (Randy Meisner) on hale
; Last Modified - Tues March 3, 1999 by S.L.Freeland, LMSAL for SSW  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
FUNCTION soonfits, file, head, extention=extention, help=help, $
     header_only=header_only, loud=loud
;+
;
; PROJECT:
;       Master's Thesis
;
; NAME: soonfits (function)
;
; PURPOSE: read one or more soon images and/or headers
;
; CALLING SEQUENCE:
;
;       result = soonfits(file, head [,extention=extention, help=help])
;
; INPUTS:
;       file - the file name of the SOONSPOT extended FITS file.
;
; OPTIONAL INPUTS: 
;       extention - the "frame" of the image file to read.  The
;                   default is to read image zero.
;
; OUTPUTS:
;       A 512x512 byte array containing the SOONSPOT image.
;
; OPTIONAL OUTPUTS:
;       head - the image header.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;       /loud - if set, print some info (default is quiet)
;       /header_only - if set, dont bother reading image (return -1)
;
; CALLS:
;       None.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Randy Meisner's Master's Thesis.
;
; PREVIOUS HISTORY:
;       Written March 23, 1997, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
; VERSION:
;       Version 1, March 23, 1997
;       Version 1.1 3-March-1999 - S.L.Freeland add /HEADER_ONLY
;-
;
ON_ERROR, 2

header_only=keyword_set(header_only)
loud=keyword_set(loud)

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'soonfits'
  RETURN, -999
ENDIF

IF(N_PARAMS() LT 1 OR N_PARAMS() GT 2) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:'
  PRINT, ''
  RETURN, -999
ENDIF

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Open the file and obtain the total number of available images.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

openr, in, file, /get_lun

check = fstat(in)

maxim = check.size/270720.0

if loud then box_message, $
 '% SOONFITS:  ' + strtrim(string(maxim), 2) + ' images available in ' + file + '.'

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Setup file pointing constant variable.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

fstep = long(270720)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Setup header and image variables.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

hdr = bytarr(80, 72)

im = bytarr(512, 512)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Check if image extension was input.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

IF(n_elements(extention) GT 0) THEN BEGIN
   
   IF(extention GT maxim) THEN BEGIN
      
      if loud then begin
         box_message,'% SOONFITS:  Extention number can not exceed maximum number' + $
         ' of images in file.'
      endif      
      close, /all
      free_lun, in
      return, -1
   ENDIF
   
ENDIF ELSE BEGIN
                                ; If no extention was input, default
                                ; to zero.
   extention = 0
   
ENDELSE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Position the file pointer to the image in the file.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pointlun = fstep*extention

point_lun, in, pointlun
if loud then $
  box_message,['% SOONFITS:  Reading image ' + strtrim(string(extention), 2)]

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Read in the image and header.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

readu, in, hdr
head = string(hdr > 32b)

if header_only then result= head else begin  ; function returns header
   readu, in, im                                  
   result=temporary(im)                      ; else return data
endelse
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Close the file.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

close, /all
free_lun, in

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Convert the header to a string array.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


RETURN, result

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'soonfits.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
